#include <string>
#include<iostream>
using namespace std;

/**
 * The abstract Coffee class defines the functionality of any Coffee
 * implemented by subclasses of Coffee
 */
class Coffee {
  public:
	  virtual double getCost() = 0;
	  virtual string getIngredients() = 0;
};
/** 
 * Kind of Coffee
 */
class Espresso : public Coffee {
  public:
	double getCost(){
		return 1.25;
	}
	string getIngredients(){
		return "Strong Coffee";
	}
};

/**
* Kind of Coffee
*/
class SimpleCoffee : public Coffee {
public:
	double getCost(){
		return 1;
	}
	string getIngredients(){
		return "Coffee";
	}
};

/**
 * Abstract decorator class - note that it extends the Coffee abstract class
 */
class CoffeeDecorator : public Coffee {
protected:
	Coffee *decoratedCoffee;
public:
	/**
	 * Wraps a Coffee object inside an object of one of
	 * CoffeeDecorator's subclasses
	 */
	CoffeeDecorator(Coffee *decoratedCoffee){
		this->decoratedCoffee = decoratedCoffee;
	}
	/**
	 * Provides the wrapper with the Coffee interface and delegates
	 * its methods to the wrapped Coffee object.
	 */
	double getCost(){
		return decoratedCoffee->getCost();
	}
	string getIngredients(){
		return decoratedCoffee->getIngredients();
	}
};

/** Decorator that mixes Milk with coffee.
 *  It is a subclass of CoffeeDecorator, and thus a subclass of Coffee.
 */
class Milk : public CoffeeDecorator {
  public:
	  /**
 	   * When creating a decorated Coffee, pass a Coffee to be decorated
	   * as a parameter. Note that this can be an already-decorated Coffee.
	   */
	  Milk(Coffee *decoratedCoffee) : CoffeeDecorator(decoratedCoffee){}
	  /**
	   * Overriding methods defined in the abstract superclass.
	   * Enables to provide different behavior for decorated Coffee methods
	   */
	  double getCost(){
		  return CoffeeDecorator::getCost() + 0.5;
	  }
	  string getIngredients(){
		  return CoffeeDecorator::getIngredients() + ", Milk";
	  }
	  /**
	   * May also add additional members for decorated-specific data
	   * or behavior
	   */
};

class Sprinkles : public CoffeeDecorator {
public:
	Sprinkles(Coffee *decoratedCoffee) : CoffeeDecorator(decoratedCoffee){}
	double getCost(){
		return CoffeeDecorator::getCost() + 0.2;
	}
	string getIngredients(){
		return CoffeeDecorator::getIngredients() + ", Sprinkles";
	}
};

class Whip : public CoffeeDecorator {
public:
	Whip(Coffee *decoratedCoffee) : CoffeeDecorator(decoratedCoffee){}
	double getCost(){
		return CoffeeDecorator::getCost() + 0.2;
	}
	string getIngredients(){
		return CoffeeDecorator::getIngredients() + ", Sprinkles";
	}
};

int main(){
	Coffee *c = new SimpleCoffee();
	cout << "Cost: " << c->getCost() << " Ingredients: " << c->getIngredients() << endl;

	c = new Milk(c);
	cout << "Cost: " << c->getCost() << " Ingredients: " << c->getIngredients() << endl;

	c = new Sprinkles(c);
	cout << "Cost: " << c->getCost() << " Ingredients: " << c->getIngredients() << endl;

	c = new Whip(c);
	cout << "Cost: " << c->getCost() << " Ingredients: " << c->getIngredients() << endl;

	// Note that you can also stack more than one decorator of the same type
	c = new Sprinkles(c);
	cout << "Cost: " << c->getCost() << " Ingredients: " << c->getIngredients() << endl;

	c = new Espresso();
	cout << "Cost: " << c->getCost() << " Ingredients: " << c->getIngredients() << endl;

	c = new Milk(c);
	cout << "Cost: " << c->getCost() << " Ingredients: " << c->getIngredients() << endl;

	system("PAUSE");
	return EXIT_SUCCESS;
}